# -*- coding: utf-8 -*-
"""
Create, Read, Update, Delete functionality

"""

from sqlalchemy import insert
from sqlalchemy.orm import Session
from fastapi.responses import JSONResponse
from . import models
from .models import RS_enum, RC_enum, timeseries_timeseries_roles_table
from .schemas import TimeseriesCreate


def get_timeseries(db: Session, timeseries_id: int):
    db_object = db.query(models.Timeseries).filter(models.Timeseries.id == timeseries_id).first()
    # there is a mismatch with additional_metadata
    if db_object:
        db_object.additional_metadata = str(db_object.additional_metadata)
    return db_object


def get_all_timeseries(db: Session, skip : int = 0, limit: int = None):
    db_objects = db.query(models.Timeseries).offset(skip).limit(limit).all()
    for db_object in db_objects:
        # there is a mismatch with additional_metadata
        db_object.additional_metadata = str(db_object.additional_metadata)
    return db_objects


def get_timeseries_by_unique_constraints(db: Session, station_id: int, variable_id: int, label: str):
    db_object = db.query(models.Timeseries).filter(models.Timeseries.station_id == station_id) \
                                      .filter(models.Timeseries.variable_id == variable_id) \
                                      .filter(models.Timeseries.label == label.strip()) \
                                      .first()
    # there is a mismatch with additional_metadata
    if db_object:
        db_object.additional_metadata = str(db_object.additional_metadata)
    return db_object


# is this internal, or should this also go to public REST api?
def get_role_ids_of_timeseries(db: Session, timeseries_id: int):
    db_objects = db.query(models.TimeseriesTimeseriesRoles) \
                                      .filter(models.TimeseriesTimeseriesRoles.timeseries_id == timeseries_id) \
                                      .all()
    return db_objects


# is this internal, or should this also go to public REST api?
def get_unique_timeseries_role(db: Session, role: int, person_id: int, status: int):
    db_object = db.query(models.TimeseriesRole).filter(models.TimeseriesRole.role == role) \
                                      .filter(models.TimeseriesRole.person_id == person_id) \
                                      .filter(models.TimeseriesRole.status == status) \
                                      .first()
    return db_object


def __get_status_enum(db: Session):
    return db.query(RS_enum).all()


def __get_role_enum(db: Session):
    return db.query(RC_enum).all()


def create_timeseries(db: Session, timeseries: TimeseriesCreate):
    enum_RS = __get_status_enum(db)
    enum_RC = __get_role_enum(db)
    timeseries_dict = timeseries.dict()
    roles_data = timeseries_dict.pop('roles', None)
    db_timeseries = models.Timeseries(**timeseries_dict)
    # there's also a mismatch with additional_metadata --> BUT: this should not be switched back!
    # in upload command, we have now: "additional_metadata": "{}"
    # but return from this method gives: "additional_metadata": {}
    # ==> there is a mismatch between model(JSONB) and schema(JSON)
    db_timeseries.additional_metadata = str(db_timeseries.additional_metadata)
    db.add(db_timeseries)
    result = db.commit()
    db.refresh(db_timeseries)
    # get timeseries_id
    timeseries_id = db_timeseries.id
    # store roles (and update associaton table?!)
    for r in roles_data:
        # try which format is used
        try:
            db_role = models.TimeseriesRole(**r)
        except:
            print ("human readable format: ", roles_data)
        # check whether role is already present in database
        db_object = get_unique_timeseries_role(db, db_role.role, db_role.person_id, db_role.status)
        if db_object:
            role_id = db_object.id
        else:
            db.add(db_role)
            db.commit()
            db.refresh(db_role)
            role_id = db_role.id
        # now into associaton table! --> This should be done automatically?!
        db.execute(insert(timeseries_timeseries_roles_table).values(timeseries_id=timeseries_id, role_id=role_id))
        db.commit()
    return db_timeseries
